@@ -82,8 +82,8 @@ binomial.fit.robmixglm <- function(x,y,offset,gh,notrials,EMTol,  calcHessian=TR
     
     # get the starting values
     prefit.coef <- coef(robust.binomial.prefit)
-    # assume 50% outliers as a starting point
-    currlpoutlier <- log(0.5/(1-0.5))
+    # assume 20% outliers as a starting point
+    currlpoutlier <- log(0.2/(1-0.2))
     currxcoef <- matrix(prefit.coef[1:(length(prefit.coef))],ncol=1)
     currxcoef <- ifelse(is.na(currxcoef),0,currxcoef)
     currtau2 <- min(rgamma(1,2,1),2)
@@ -109,8 +109,12 @@ binomial.fit.robmixglm <- function(x,y,offset,gh,notrials,EMTol,  calcHessian=TR
       ll1 <- dbinom(y[,1], y[,1]+y[,2], logit(lp), log = TRUE)+log(1-currpoutlier) 
       ll2 <- llrandbinomcpp(y, lp, currtau2, gh)+log(currpoutlier)
       
-      l <- exp(cbind(ll1,ll2))
-      prop <- l/apply(l,1,sum)
+      ll <- cbind(ll1,ll2)
+      prop <- t(apply(ll,1,function(x) {
+        x <- x-max(x)
+        x <- ifelse(x==-Inf,-1e100,x)
+        return(exp(x)/sum(exp(x)))
+      }))
       # calculate outlier proportion
       poutlier <- sum(prop[,2])/dim(prop)[1]
       currlpoutlier <- log(poutlier/(1-poutlier))
@@ -167,9 +171,9 @@ binomial.fit.robmixglm <- function(x,y,offset,gh,notrials,EMTol,  calcHessian=TR
     
     for (i in 1:notrials) {
       try({
-        outliers <- rbinom(dim(x)[1],1,0.5)
-        #outliers <- sample(c(rep(0,dim(x)[1] %/% 2),rep(1,dim(x)[1]-dim(x)[1] %/% 2)),dim(x)[1])
-       thefit <- suppressWarnings(fitonemlreg(y,outliers,x,offset,fixed=NULL))
+        noutliers <- max(1,round(dim(x)[1]*0.2))
+        outliers <- sample(c(rep(1,noutliers),rep(0,dim(x)[1]-noutliers)),dim(x)[1])
+        thefit <- fitonemlreg(y,outliers,x,offset,fixed=NULL)
       if (verbose) print(c(thefit$ll,thefit$start.val))
       if (thefit$ll>maxll) {
         maxll <- thefit$ll
@@ -179,7 +183,7 @@ binomial.fit.robmixglm <- function(x,y,offset,gh,notrials,EMTol,  calcHessian=TR
       if (!is.finite(maxll)) stop("No starting values found") 
     }
     } else {
-    thefit <- suppressWarnings(fitonemlreg(y,NULL,x,offset,fixed=NULL))
+    thefit <- fitonemlreg(y,NULL,x,offset,fixed=NULL)
     start.val <- thefit$start.val
   }
   
@@ -219,8 +223,12 @@ binomial.fit.robmixglm <- function(x,y,offset,gh,notrials,EMTol,  calcHessian=TR
   ll1 <- dbinom(y[,1], y[,1]+y[,2], logit(lp), log = TRUE)+log(1-poutlier) 
   ll2 <- llrandbinomcpp(y, lp, tau2, gh)+log(poutlier)
   
-  l <- exp(cbind(ll1,ll2))
-  prop <- l/apply(l,1,sum)
+  ll <- cbind(ll1,ll2)
+  prop <- t(apply(ll,1,function(x) {
+    x <- x-max(x)
+    x <- ifelse(x==-Inf,-1e100,x)
+    return(exp(x)/sum(exp(x)))
+  }))
   
   coef.names <- c(dimnames(x)[[2]],"Outlier p.","Tau-sq")
   return(list(fit=robustbinomial.fit,prop=prop,logLik=-robustbinomial.fit@min,np=length(coef.names),nobs=dim(x)[1],coef.names=coef.names))  
